import jwo.processing.utils.*;    // For the standard ellipse class in Jo Wood's ProcessingUtils package.

// Sketch to demonstrate the use of the StandardEllipse class. Draws some random points
// then finds their standard ellipse and draws it.
// Version 1.1, 24th June, 2009.
// Author Jo Wood.

// ------------------ Object variables --------------------

ArrayList points;
static final int NUM_POINTS = 50;
static final int HIGH_WEIGHT = 10;

// ------------------- Initialisation ---------------------

/** Creates a set of random points from which the standard ellipses will be created.
  */
void setup()
{
  size(600,600);
  smooth();

  points = new ArrayList();
  
  for (int i=0; i<NUM_POINTS; i++)
  {  
    if (random(1) < 0.5)
    {
      // Add a highly weighted point somewhere above-left of centre.
      points.add(new PVector(width/6 + random(width/6),height/3+random(-height/10,height/10),HIGH_WEIGHT));
    }
    else
    {
      // Add a unit weighted point anywhere in the central belt of the region.
      points.add(new PVector(width/2 + random(-width/3,width/3),height/2+random(-height/6,height/6)));
    }
  }
}

// ------------------ Processing draw --------------------

/** Draws the points, randomly perturbs them and draws their standard ellipse.
 */
void draw()
{
  background(255);
  fill(30,30,100,50);
  strokeWeight(3);
    
  // Plot the original points and make them move about a bit.
  Iterator i = points.iterator();
  while (i.hasNext())
  {
    PVector p = (PVector)i.next();
    p.x += random(-2,2);
    p.y += random(-2,2);
    
    if (p.z == 0)
    {
      stroke(0);
    }
    else
    {
      stroke(200,50,50);
    }    
    point(p.x,p.y);
  }
  
  // Calculate the standard ellipse of the points.
  StandardEllipse standardEllipse = new StandardEllipse(points);
  
  // Draw the unweighted version of the ellipse.
  stroke(30,30,100);
  standardEllipse.draw(this);
  
  // Draw the ellipse axes.
  stroke(80,80,80,100);
  strokeWeight(1);
  standardEllipse.drawAxes(this);
  
   // Draw the weighted version of the ellipse.
  standardEllipse.setIsWeighted(true);
  stroke(30,100,30);
  standardEllipse.draw(this);
  
  // Draw the ellipse axes.
  stroke(80,80,80,100);
  strokeWeight(1);
  standardEllipse.drawAxes(this);
  
}
